@@ -6,20 +6,45 @@ Pyramid authn policy that ties together multiple backends.
 """
 
 __ver_major__ = 0
-__ver_minor__ = 2
+__ver_minor__ = 3
 __ver_patch__ = 0
 __ver_sub__ = ""
 __ver_tuple__ = (__ver_major__, __ver_minor__, __ver_patch__, __ver_sub__)
 __version__ = "%d.%d.%d%s" % __ver_tuple__
 
 
-from zope.interface import implements
+import sys
+
+from zope.interface import implementer
 
 from pyramid.interfaces import IAuthenticationPolicy, PHASE2_CONFIG
 from pyramid.security import Everyone, Authenticated
 from pyramid.authorization import ACLAuthorizationPolicy
 
 
+if sys.version_info > (3,):  # pragma: nocover
+    basestring = str
+
+
+class MultiAuthPolicySelected(object):
+    """Event for tracking which authentication policy was used.
+
+    This event is fired whenever a particular backend policy is successfully
+    used for authentication.  It can be used by other parts of the code in
+    order to act based on the selected policy::
+
+        from pyramid.events import subscriber
+
+        @subscriber(MultiAuthPolicySelected)
+        def track_policy(event):
+            print("We selected policy %s" % event.policy)
+
+    """
+    def __init__(self, policy):
+        self.policy = policy
+
+
+@implementer(IAuthenticationPolicy)
 class MultiAuthenticationPolicy(object):
     """Pyramid authentication policy for stacked authentication.
 
@@ -36,8 +61,6 @@ class MultiAuthenticationPolicy(object):
 
     """
 
-    implements(IAuthenticationPolicy)
-
     def __init__(self, policies, callback=None):
         self._policies = policies
         self._callback = callback
@@ -54,6 +77,8 @@ class MultiAuthenticationPolicy(object):
         for policy in self._policies:
             userid = policy.authenticated_userid(request)
             if userid is not None:
+                request.registry.notify(MultiAuthPolicySelected(policy))
+
                 if self._callback is None:
                     break
                 if self._callback(userid, request) is not None:
@@ -283,9 +308,10 @@ def get_policy_definitions(settings):
     into a dict indexed by the policy name.
     """
     policy_definitions = {}
-    for name, value in settings.iteritems():
+    for name in settings:
         if not name.startswith("multiauth.policy."):
             continue
+        value = settings[name]
         name = name[len("multiauth.policy."):]
         policy_name, setting_name = name.split(".", 1)
         if policy_name not in policy_definitions:
